package com.sun.java.util.jar.pack;

import java.io.SequenceInputStream;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.ByteArrayOutputStream;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.ListIterator;
import java.util.Comparator;
import java.util.Collections;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.Collection;
import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;

class Package
{
    int verbose;
    final int magic = -889270259;
    int default_modtime;
    int default_options;
    Package.Version defaultClassVersion;
    final Package.Version minClassVersion;
    final Package.Version maxClassVersion;
    final Package.Version packageVersion;
    Package.Version observedHighestClassVersion;
    ConstantPool.IndexGroup cp;
    public static final Attribute.Layout attrCodeEmpty;
    public static final Attribute.Layout attrBootstrapMethodsEmpty;
    public static final Attribute.Layout attrInnerClassesEmpty;
    public static final Attribute.Layout attrSourceFileSpecial;
    public static final Map<Attribute.Layout, Attribute> attrDefs;
    ArrayList<Package.Class> classes;
    ArrayList<Package.File> files;
    List<Package.InnerClass> allInnerClasses;
    Map<ConstantPool.ClassEntry, Package.InnerClass> allInnerClassesByThis;
    private static final int SLASH_MIN = 46;
    private static final int SLASH_MAX = 47;
    private static final int DOLLAR_MIN = 0;
    private static final int DOLLAR_MAX = 45;
    static final List<Object> noObjects;
    static final List<Package.Class.Field> noFields;
    static final List<Package.Class.Method> noMethods;
    static final List<Package.InnerClass> noInnerClasses;
    
    public Package() {
        PropMap currentPropMap = Utils.currentPropMap();
        if (currentPropMap != null) {
            this.verbose = currentPropMap.getInteger("com.sun.java.util.jar.pack.verbose");
        }
        this.default_modtime = 0;
        this.default_options = 0;
        this.defaultClassVersion = null;
        this.observedHighestClassVersion = null;
        this.cp = new ConstantPool.IndexGroup();
        this.classes = new ArrayList();
        this.files = new ArrayList();
        this.allInnerClasses = new ArrayList();
        this.minClassVersion = Constants.JAVA_MIN_CLASS_VERSION;
        this.maxClassVersion = Constants.JAVA_MAX_CLASS_VERSION;
        this.packageVersion = null;
    }
    
    public Package(Package.Version version, Package.Version version2, Package.Version packageVersion) {
        PropMap currentPropMap = Utils.currentPropMap();
        if (currentPropMap != null) {
            this.verbose = currentPropMap.getInteger("com.sun.java.util.jar.pack.verbose");
        }
        this.default_modtime = 0;
        this.default_options = 0;
        this.defaultClassVersion = null;
        this.observedHighestClassVersion = null;
        this.cp = new ConstantPool.IndexGroup();
        this.classes = new ArrayList();
        this.files = new ArrayList();
        this.allInnerClasses = new ArrayList();
        this.minClassVersion = ((version == null) ? Constants.JAVA_MIN_CLASS_VERSION : version);
        this.maxClassVersion = ((version2 == null) ? Constants.JAVA_MAX_CLASS_VERSION : version2);
        this.packageVersion = packageVersion;
    }
    
    public void reset() {
        this.cp = new ConstantPool.IndexGroup();
        this.classes.clear();
        this.files.clear();
        BandStructure.nextSeqForDebug = 0;
        this.observedHighestClassVersion = null;
    }
    
    Package.Version getDefaultClassVersion() {
        return this.defaultClassVersion;
    }
    
    private void setHighestClassVersion() {
        if (this.observedHighestClassVersion != null) {
            return;
        }
        Package.Version java_MIN_CLASS_VERSION = Constants.JAVA_MIN_CLASS_VERSION;
        Iterator<Package.Class> iterator = this.classes.iterator();
        while (iterator.hasNext()) {
            Package.Version version = ((Package.Class)iterator.next()).getVersion();
            if (java_MIN_CLASS_VERSION.lessThan(version)) {
                java_MIN_CLASS_VERSION = version;
            }
            else {
                continue;
            }
        }
        this.observedHighestClassVersion = java_MIN_CLASS_VERSION;
        return;
    }
    
    Package.Version getHighestClassVersion() {
        this.setHighestClassVersion();
        return this.observedHighestClassVersion;
    }
    
    public List<Package.Class> getClasses() {
        return this.classes;
    }
    
    void addClass(Package.Class e) {
        assert e.getPackage() == this;
        int add = this.classes.add(e);
        assert !(add == 0);
        if (e.file == null) {
            e.initFile(null);
        }
        this.addFile(e.file);
        return;
    }
    
    public List<Package.File> getFiles() {
        return this.files;
    }
    
    public List<Package.File> getClassStubs() {
        ArrayList list = new ArrayList(this.classes.size());
        for (Package.Class class1 : this.classes) {
            assert !(!class1.file.isClassStub());
            list.add(class1.file);
        }
        return list;
    }
    
    Package.File newStub(String s) {
        Package.File file = new Package.File(s);
        file.options |= 0x2;
        file.prepend = null;
        file.append = null;
        return file;
    }
    
    private static String fixupFileName(String s) {
        String replace = s.replace(java.io.File.separatorChar, '/');
        if (replace.startsWith("/")) {
            throw new IllegalArgumentException("absolute file name " + replace);
        }
        return replace;
    }
    
    void addFile(Package.File e) {
        int add = this.files.add(e);
        assert !(add == 0);
        return;
    }
    
    public List<Package.InnerClass> getAllInnerClasses() {
        return this.allInnerClasses;
    }
    
    public void setAllInnerClasses(Collection<Package.InnerClass> collection) {
        assert collection != this.allInnerClasses;
        this.allInnerClasses.clear();
        this.allInnerClasses.addAll(collection);
        this.allInnerClassesByThis = new HashMap(this.allInnerClasses.size());
        for (Package.InnerClass innerClass : this.allInnerClasses) {
            Object put = this.allInnerClassesByThis.put(innerClass.thisClass, innerClass);
            assert put == null;
            continue;
        }
        return;
    }
    
    public Package.InnerClass getGlobalInnerClass(ConstantPool.Entry entry) {
        assert !(!(entry instanceof ConstantPool.ClassEntry));
        return (Package.InnerClass)this.allInnerClassesByThis.get(entry);
    }
    
    private static void visitInnerClassRefs(Collection<Package.InnerClass> collection, int n, Collection<ConstantPool.Entry> collection2) {
        if (collection == null) {
            return;
        }
        if (n == 0) {
            collection2.add(getRefString("InnerClasses"));
        }
        if (collection.size() > 0) {
            Iterator<Object> iterator = collection.iterator();
            while (iterator.hasNext()) {
                ((Package.InnerClass)iterator.next()).visitRefs(n, collection2);
            }
        }
        return;
    }
    
    static String[] parseInnerClassName(String s) {
        int length = s.length();
        int n = lastIndexOf(46, 47, s, s.length()) + 1;
        int n2 = lastIndexOf(0, 45, s, s.length());
        if (n2 < n) {
            return null;
        }
        String s2;
        String s3;
        if (isDigitString(s, n2 + 1, length)) {
            s2 = s.substring(n2 + 1, length);
            s3 = null;
        }
        else if ((n2 = lastIndexOf(0, 45, s, n2 - 1)) > n && isDigitString(s, n2 + 1, n2)) {
            s2 = s.substring(n2 + 1, n2);
            s3 = s.substring(n2 + 1, length).intern();
        }
        else {
            s2 = null;
            s3 = s.substring(n2 + 1, length).intern();
        }
        String intern;
        if (s2 == null) {
            intern = s.substring(0, n2).intern();
        }
        else {
            intern = null;
        }
        return new String[] { intern, s2, s3 };
    }
    
    private static int lastIndexOf(int n, int n2, String s, int index) {
        while (--index >= 0) {
            int char1 = s.charAt(index);
            if (char1 >= n && char1 <= n2) {
                return index;
            }
            continue;
        }
        return -1;
    }
    
    private static boolean isDigitString(String s, int i, int n) {
        if (i == n) {
            return false;
        }
        while (i < n) {
            int char1 = s.charAt(i);
            if (char1 < '0' || char1 > '9') {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    static String getObviousSourceFile(String s) {
        String substring = s.substring(lastIndexOf(46, 47, s, s.length()) + 1);
        int length = substring.length();
        while (true) {
            int lastIndex = lastIndexOf(0, 45, substring, length - 1);
            if (lastIndex < 0) {
                break;
            }
            length = lastIndex;
            if (length == 0) {
                break;
            }
            continue;
        }
        return substring.substring(0, length) + ".java";
    }
    
    static ConstantPool.Utf8Entry getRefString(String s) {
        return ConstantPool.getUtf8Entry(s);
    }
    
    static ConstantPool.LiteralEntry getRefLiteral(Comparable<?> comparable) {
        return ConstantPool.getLiteralEntry(comparable);
    }
    
    void stripAttributeKind(String s) {
        if (this.verbose > 0) {
            Utils.log.info("Stripping " + s.toLowerCase() + " data and attributes...");
        }
        int n = -1;
        switch (s.hashCode()) {
            case 65906227: {
                if (s.equals("Debug")) {
                    n = 0;
                    break;
                }
                break;
            }
            case -1679822317: {
                if (s.equals("Compile")) {
                    n = 1;
                    break;
                }
                break;
            }
            case 679220772: {
                if (s.equals("Exceptions")) {
                    n = 2;
                    break;
                }
                break;
            }
            case -503167036: {
                if (s.equals("Constant")) {
                    n = 3;
                    break;
                }
                break;
            }
        }
        switch (n) {
            case 0: {
                this.strip("SourceFile");
                this.strip("LineNumberTable");
                this.strip("LocalVariableTable");
                this.strip("LocalVariableTypeTable");
                break;
            }
            case 1: {
                this.strip("Deprecated");
                this.strip("Synthetic");
                break;
            }
            case 2: {
                this.strip("Exceptions");
                break;
            }
            case 3: {
                this.stripConstantFields();
                break;
            }
        }
    }
    
    public void trimToSize() {
        this.classes.trimToSize();
        Iterator<Package.Class> iterator = this.classes.iterator();
        while (iterator.hasNext()) {
            ((Package.Class)iterator.next()).trimToSize();
        }
        this.files.trimToSize();
    }
    
    public void strip(String s) {
        Iterator<Package.Class> iterator = this.classes.iterator();
        while (iterator.hasNext()) {
            ((Package.Class)iterator.next()).strip(s);
        }
    }
    
    public void stripConstantFields() {
        Iterator<Package.Class> iterator = this.classes.iterator();
        while (iterator.hasNext()) {
            Iterator<Package.Class.Field> iterator2 = ((Package.Class)iterator.next()).fields.iterator();
            while (iterator2.hasNext()) {
                Package.Class.Field field = (Package.Class.Field)iterator2.next();
                if (Modifier.isFinal(field.flags) && Modifier.isStatic(field.flags) && field.getAttribute("ConstantValue") != null && !field.getName().startsWith("serial") && this.verbose > 2) {
                    Utils.log.fine(">> Strip " + this + " ConstantValue");
                    iterator2.remove();
                }
                else {
                    continue;
                }
            }
        }
    }
    
    protected void visitRefs(int n, Collection<ConstantPool.Entry> collection) {
        Iterator<Package.Class> iterator = this.classes.iterator();
        while (iterator.hasNext()) {
            ((Package.Class)iterator.next()).visitRefs(n, collection);
        }
        if (n != 0) {
            Iterator<Package.File> iterator2 = this.files.iterator();
            while (iterator2.hasNext()) {
                ((Package.File)iterator2.next()).visitRefs(n, collection);
            }
            visitInnerClassRefs(this.allInnerClasses, n, collection);
        }
    }
    
    void reorderFiles(boolean b, boolean b2) {
        if (!b) {
            Collections.sort(this.classes);
        }
        List<Package.File> classStubs = this.getClassStubs();
        Iterator<Package.File> iterator = this.files.iterator();
        while (iterator.hasNext()) {
            Package.File file = (Package.File)iterator.next();
            if (file.isClassStub() || (b2 && file.isDirectory())) {
                iterator.remove();
            }
            else {
                continue;
            }
        }
        Collections.sort(this.files, new Comparator<Package.File>() {
            @Override
            public int compare(Package.File file, Package.File file2) {
                String nameString = file.nameString;
                String nameString2 = file2.nameString;
                if (nameString.equals(nameString2)) {
                    return 0;
                }
                if ("META-INF/MANIFEST.MF".equals(nameString)) {
                    return -1;
                }
                if ("META-INF/MANIFEST.MF".equals(nameString2)) {
                    return 1;
                }
                String substring = nameString.substring(1 + nameString.lastIndexOf(47));
                String substring2 = nameString2.substring(1 + nameString2.lastIndexOf(47));
                int compareTo = substring.substring(1 + substring.lastIndexOf(46)).compareTo(substring2.substring(1 + substring2.lastIndexOf(46)));
                if (compareTo != 0) {
                    return compareTo;
                }
                return nameString.compareTo(nameString2);
            }
        });
        this.files.addAll(classStubs);
    }
    
    void trimStubs() {
        ListIterator<Package.File> listIterator = this.files.listIterator(this.files.size());
        while (listIterator.hasPrevious()) {
            Package.File file = (Package.File)listIterator.previous();
            if (!file.isTrivialClassStub()) {
                if (this.verbose > 1) {
                    Utils.log.fine("Keeping last non-trivial " + file);
                    break;
                }
                break;
            }
            else {
                if (this.verbose > 2) {
                    Utils.log.fine("Removing trivial " + file);
                }
                listIterator.remove();
            }
        }
        if (this.verbose > 0) {
            Utils.log.info("Transmitting " + this.files.size() + " files, including per-file data for " + this.getClassStubs().size() + " classes out of " + this.classes.size());
        }
    }
    
    void buildGlobalConstantPool(Set<ConstantPool.Entry> set) {
        if (this.verbose > 1) {
            Utils.log.fine("Checking for unused CP entries");
        }
        set.add(getRefString(""));
        this.visitRefs(1, set);
        ConstantPool.completeReferencesIn(set, false);
        if (this.verbose > 1) {
            Utils.log.fine("Sorting CP entries");
        }
        ConstantPool.Index[] partitionByTag = ConstantPool.partitionByTag(ConstantPool.makeIndex("unsorted", set));
        for (int i = 0; i < ConstantPool.TAGS_IN_ORDER.length; ++i) {
            int n = ConstantPool.TAGS_IN_ORDER[i];
            ConstantPool.Index index = partitionByTag[n];
            if (index != null) {
                ConstantPool.sort(index);
                this.cp.initIndexByTag((byte)n, index);
                partitionByTag[n] = null;
            }
        }
        int j = 0;
        while (j < partitionByTag.length) {
            ConstantPool.Index index2 = partitionByTag[j];
            assert index2 == null;
            ++j;
        }
        int k = 0;
        while (k < ConstantPool.TAGS_IN_ORDER.length) {
            ConstantPool.Index indexByTag = this.cp.getIndexByTag(ConstantPool.TAGS_IN_ORDER[k]);
            assert !(!indexByTag.assertIsSorted());
            if (this.verbose > 2) {
                Utils.log.fine(indexByTag.dumpString());
            }
            ++k;
        }
    }
    
    void ensureAllClassFiles() {
        HashSet set = new HashSet(this.files);
        for (Package.Class class1 : this.classes) {
            if (!set.contains(class1.file)) {
                this.files.add(class1.file);
            }
            else {
                continue;
            }
        }
    }
    
    static void access$000(Collection collection, int n, Collection collection2) {
        visitInnerClassRefs(collection, n, collection2);
    }
    
    static String access$100(String s) {
        return fixupFileName(s);
    }
    
    static {
        HashMap m = new HashMap(3);
        attrCodeEmpty = Attribute.define(m, 2, "Code", "").layout();
        attrBootstrapMethodsEmpty = Attribute.define(m, 0, "BootstrapMethods", "").layout();
        attrInnerClassesEmpty = Attribute.define(m, 0, "InnerClasses", "").layout();
        attrSourceFileSpecial = Attribute.define(m, 0, "SourceFile", "RUNH").layout();
        attrDefs = Collections.unmodifiableMap(m);
        assert lastIndexOf(0, 45, "x$$y$", 4) == 2;
        assert lastIndexOf(46, 47, "x//y/", 4) == 2;
        noObjects = Arrays.asList(new Object[0]);
        noFields = Arrays.asList(new Package.Class.Field[0]);
        noMethods = Arrays.asList(new Package.Class.Method[0]);
        noInnerClasses = Arrays.asList(new Package.InnerClass[0]);
        return;
    }
    
    public final class Class extends Attribute.Holder implements Comparable<Package.Class>
    {
        Package.File file;
        int magic;
        Package.Version version;
        ConstantPool.Entry[] cpMap;
        ConstantPool.ClassEntry thisClass;
        ConstantPool.ClassEntry superClass;
        ConstantPool.ClassEntry[] interfaces;
        ArrayList<Package.Class.Field> fields;
        ArrayList<Package.Class.Method> methods;
        ArrayList<Package.InnerClass> innerClasses;
        ArrayList<ConstantPool.BootstrapMethodEntry> bootstrapMethods;
        
        public Package getPackage() {
            return Package.this;
        }
        
        Class(int flags, ConstantPool.ClassEntry thisClass, ConstantPool.ClassEntry superClass, ConstantPool.ClassEntry[] interfaces) {
            this.magic = -889275714;
            this.version = Package.this.defaultClassVersion;
            this.flags = flags;
            this.thisClass = thisClass;
            this.superClass = superClass;
            this.interfaces = interfaces;
            int add = Package.this.classes.add(this);
            assert !(add == 0);
            return;
        }
        
        Class(String s) {
            this.initFile(Package.this.newStub(s));
        }
        
        List<Package.Class.Field> getFields() {
            return (this.fields == null) ? Package.noFields : this.fields;
        }
        
        List<Package.Class.Method> getMethods() {
            return (this.methods == null) ? Package.noMethods : this.methods;
        }
        
        public String getName() {
            return this.thisClass.stringValue();
        }
        
        Package.Version getVersion() {
            return this.version;
        }
        
        @Override
        public int compareTo(Package.Class class1) {
            return this.getName().compareTo(class1.getName());
        }
        
        String getObviousSourceFile() {
            return Package.getObviousSourceFile(this.getName());
        }
        
        private void transformSourceFile(boolean b) {
            Attribute attribute = this.getAttribute(Package.attrSourceFileSpecial);
            if (attribute == null) {
                return;
            }
            String obviousSourceFile = this.getObviousSourceFile();
            ArrayList list = new ArrayList(1);
            attribute.visitRefs(this, 1, list);
            ConstantPool.Utf8Entry utf8Entry = (ConstantPool.Utf8Entry)list.get(0);
            if (utf8Entry == null) {
                if (b) {
                    attribute = Attribute.find(0, "SourceFile", "H").addContent(new byte[2]);
                }
                else {
                    byte[] array = new byte[2];
                    attribute = Package.attrSourceFileSpecial.addContent(array, Fixups.addRefWithBytes(null, array, Package.getRefString(obviousSourceFile)));
                }
            }
            else if (obviousSourceFile.equals(utf8Entry.stringValue())) {
                if (b) {
                    attribute = Package.attrSourceFileSpecial.addContent(new byte[2]);
                }
                else
                    assert false;
            }
            if (attribute != attribute) {
                if (Package.this.verbose > 2) {
                    Utils.log.fine("recoding obvious SourceFile=" + obviousSourceFile);
                }
                ArrayList attributes = new ArrayList(this.getAttributes());
                attributes.set(attributes.indexOf(attribute), attribute);
                this.setAttributes(attributes);
            }
            return;
        }
        
        void minimizeSourceFile() {
            this.transformSourceFile(true);
        }
        
        void expandSourceFile() {
            this.transformSourceFile(false);
        }
        
        @Override
        protected ConstantPool.Entry[] getCPMap() {
            return this.cpMap;
        }
        
        protected void setCPMap(ConstantPool.Entry[] cpMap) {
            this.cpMap = cpMap;
        }
        
        boolean hasBootstrapMethods() {
            return this.bootstrapMethods != null && !this.bootstrapMethods.isEmpty();
        }
        
        List<ConstantPool.BootstrapMethodEntry> getBootstrapMethods() {
            return this.bootstrapMethods;
        }
        
        ConstantPool.BootstrapMethodEntry[] getBootstrapMethodMap() {
            return this.hasBootstrapMethods() ? ((ConstantPool.BootstrapMethodEntry[])this.bootstrapMethods.toArray(new ConstantPool.BootstrapMethodEntry[this.bootstrapMethods.size()])) : null;
        }
        
        void setBootstrapMethods(Collection<ConstantPool.BootstrapMethodEntry> c) {
            assert this.bootstrapMethods == null;
            this.bootstrapMethods = new ArrayList(c);
            return;
        }
        
        boolean hasInnerClasses() {
            return this.innerClasses != null;
        }
        
        List<Package.InnerClass> getInnerClasses() {
            return this.innerClasses;
        }
        
        public void setInnerClasses(Collection<Package.InnerClass> c) {
            this.innerClasses = ((c == null) ? null : new ArrayList(c));
            Attribute attribute = this.getAttribute(Package.attrInnerClassesEmpty);
            if (this.innerClasses != null && attribute == null) {
                this.addAttribute(Package.attrInnerClassesEmpty.canonicalInstance());
            }
            else if (this.innerClasses == null && attribute != null) {
                this.removeAttribute(attribute);
            }
        }
        
        public List<Package.InnerClass> computeGloballyImpliedICs() {
            HashSet<ConstantPool.Entry> set = new HashSet();
            ArrayList<Object> innerClasses = this.innerClasses;
            this.innerClasses = null;
            this.visitRefs(0, set);
            this.innerClasses = innerClasses;
            ConstantPool.completeReferencesIn(set, true);
            HashSet<ConstantPool.Entry> set2 = new HashSet();
            for (ConstantPool.Entry outerClass : set) {
                if (!(outerClass instanceof ConstantPool.ClassEntry)) {
                    continue;
                }
                while (outerClass != null) {
                    Package.InnerClass globalInnerClass = Package.this.getGlobalInnerClass(outerClass);
                    if (globalInnerClass == null) {
                        break;
                    }
                    if (!set2.add(outerClass)) {
                        break;
                    }
                    outerClass = globalInnerClass.outerClass;
                }
            }
            ArrayList o = new ArrayList();
            for (Package.InnerClass innerClass : Package.this.allInnerClasses) {
                if (set2.contains(innerClass.thisClass) || innerClass.outerClass == this.thisClass) {
                    if (Package.this.verbose > 1) {
                        Utils.log.fine("Relevant IC: " + innerClass);
                    }
                    o.add(innerClass);
                }
                else {
                    continue;
                }
            }
            return o;
        }
        
        private List<Package.InnerClass> computeICdiff() {
            List<Package.InnerClass> computeGloballyImpliedICs = this.computeGloballyImpliedICs();
            List<Object> c = this.getInnerClasses();
            if (c == null) {
                c = Collections.emptyList();
            }
            if (c.isEmpty()) {
                return computeGloballyImpliedICs;
            }
            if (computeGloballyImpliedICs.isEmpty()) {
                return c;
            }
            HashSet o = new HashSet(c);
            o.retainAll(new HashSet(computeGloballyImpliedICs));
            computeGloballyImpliedICs.addAll(c);
            computeGloballyImpliedICs.removeAll(o);
            return computeGloballyImpliedICs;
        }
        
        void minimizeLocalICs() {
            List<Package.InnerClass> computeICdiff = this.computeICdiff();
            ArrayList<Object> innerClasses = this.innerClasses;
            List<Package.InnerClass> emptyList;
            if (computeICdiff.isEmpty()) {
                emptyList = null;
                if (innerClasses != null && innerClasses.isEmpty() && Package.this.verbose > 0) {
                    Utils.log.info("Warning: Dropping empty InnerClasses attribute from " + this);
                }
            }
            else if (innerClasses == null) {
                emptyList = Collections.emptyList();
            }
            else {
                emptyList = computeICdiff;
            }
            this.setInnerClasses(emptyList);
            if (Package.this.verbose > 1 && emptyList != null) {
                Utils.log.fine("keeping local ICs in " + this + ": " + emptyList);
            }
        }
        
        int expandLocalICs() {
            ArrayList<Object> innerClasses = this.innerClasses;
            List<Package.InnerClass> computeICdiff;
            int n;
            if (innerClasses == null) {
                List<Package.InnerClass> computeGloballyImpliedICs = this.computeGloballyImpliedICs();
                if (computeGloballyImpliedICs.isEmpty()) {
                    computeICdiff = null;
                    n = 0;
                }
                else {
                    computeICdiff = computeGloballyImpliedICs;
                    n = 1;
                }
            }
            else if (innerClasses.isEmpty()) {
                computeICdiff = null;
                n = 0;
            }
            else {
                computeICdiff = this.computeICdiff();
                n = (computeICdiff.containsAll(innerClasses) ? 1 : -1);
            }
            this.setInnerClasses(computeICdiff);
            return n;
        }
        
        @Override
        public void trimToSize() {
            super.trimToSize();
            for (int i = 0; i <= 1; ++i) {
                ArrayList<Object> list = (i == 0) ? this.fields : this.methods;
                if (list != null) {
                    list.trimToSize();
                    Iterator<Object> iterator = list.iterator();
                    while (iterator.hasNext()) {
                        ((Package.Class.Member)iterator.next()).trimToSize();
                    }
                }
            }
            if (this.innerClasses != null) {
                this.innerClasses.trimToSize();
            }
        }
        
        @Override
        public void strip(String anObject) {
            if ("InnerClass".equals(anObject)) {
                this.innerClasses = null;
            }
            for (int i = 0; i <= 1; ++i) {
                ArrayList<Object> list = (i == 0) ? this.fields : this.methods;
                if (list != null) {
                    Iterator<Object> iterator = list.iterator();
                    while (iterator.hasNext()) {
                        ((Package.Class.Member)iterator.next()).strip(anObject);
                    }
                }
            }
            super.strip(anObject);
        }
        
        @Override
        protected void visitRefs(int n, Collection<ConstantPool.Entry> collection) {
            if (Package.this.verbose > 2) {
                Utils.log.fine("visitRefs " + this);
            }
            collection.add(this.thisClass);
            collection.add(this.superClass);
            collection.addAll(Arrays.asList(this.interfaces));
            for (int i = 0; i <= 1; ++i) {
                ArrayList<Object> list = (i == 0) ? this.fields : this.methods;
                if (list != null) {
                    for (Package.Class.Member obj : list) {
                        boolean n2 = false;
                        try {
                            obj.visitRefs(n, collection);
                            n2 = true;
                        }
                        finally {
                            if (n2 == 0) {
                                Utils.log.warning("Error scanning " + obj);
                            }
                        }
                    }
                }
            }
            this.visitInnerClassRefs(n, collection);
            super.visitRefs(n, collection);
        }
        
        protected void visitInnerClassRefs(int n, Collection<ConstantPool.Entry> collection) {
            Package.access$000(this.innerClasses, n, collection);
        }
        
        void finishReading() {
            this.trimToSize();
            this.maybeChooseFileName();
        }
        
        public void initFile(Package.File stub) {
            assert this.file == null;
            if (stub == null) {
                stub = Package.this.newStub(this.canonicalFileName());
            }
            this.file = stub;
            assert !(!stub.isClassStub());
            (stub.stubClass = this).maybeChooseFileName();
            return;
        }
        
        public void maybeChooseFileName() {
            if (this.thisClass == null) {
                return;
            }
            String canonicalFileName = this.canonicalFileName();
            if (this.file.nameString.equals("")) {
                this.file.nameString = canonicalFileName;
            }
            if (this.file.nameString.equals(canonicalFileName)) {
                this.file.name = Package.getRefString("");
                return;
            }
            if (this.file.name == null) {
                this.file.name = Package.getRefString(this.file.nameString);
            }
            return;
        }
        
        public String canonicalFileName() {
            if (this.thisClass == null) {
                return null;
            }
            return this.thisClass.stringValue() + ".class";
        }
        
        public java.io.File getFileName(java.io.File parent) {
            String s = this.file.name.stringValue();
            if (s.equals("")) {
                s = this.canonicalFileName();
            }
            return new java.io.File(parent, s.replace('/', java.io.File.separatorChar));
        }
        
        public java.io.File getFileName() {
            return this.getFileName(null);
        }
        
        @Override
        public String toString() {
            return this.thisClass.stringValue();
        }
        
        public abstract class Member extends Attribute.Holder implements Comparable<Package.Class.Member>
        {
            ConstantPool.DescriptorEntry descriptor;
            
            protected Member(int flags, ConstantPool.DescriptorEntry descriptor) {
                this.flags = flags;
                this.descriptor = descriptor;
            }
            
            public Package.Class thisClass() {
                return Package.Class.this;
            }
            
            public ConstantPool.DescriptorEntry getDescriptor() {
                return this.descriptor;
            }
            
            public String getName() {
                return this.descriptor.nameRef.stringValue();
            }
            
            public String getType() {
                return this.descriptor.typeRef.stringValue();
            }
            
            @Override
            protected ConstantPool.Entry[] getCPMap() {
                return Package.Class.this.cpMap;
            }
            
            @Override
            protected void visitRefs(int n, Collection<ConstantPool.Entry> collection) {
                if (Package.this.verbose > 2) {
                    Utils.log.fine("visitRefs " + this);
                }
                if (n == 0) {
                    collection.add(this.descriptor.nameRef);
                    collection.add(this.descriptor.typeRef);
                }
                else {
                    collection.add(this.descriptor);
                }
                super.visitRefs(n, collection);
            }
            
            @Override
            public String toString() {
                return Package.Class.this + "." + this.descriptor.prettyString();
            }
        }
        
        public class Field extends Package.Class.Member
        {
            int order;
            
            public Field(int n, ConstantPool.DescriptorEntry descriptorEntry) {
                super(n, descriptorEntry);
                assert !descriptorEntry.isMethod();
                if (Package.Class.this.fields == null) {
                    Package.Class.this.fields = new ArrayList();
                }
                int add = Package.Class.this.fields.add(this);
                assert !(add == 0);
                this.order = Package.Class.this.fields.size();
                return;
            }
            
            public byte getLiteralTag() {
                return this.descriptor.getLiteralTag();
            }
            
            @Override
            public int compareTo(Package.Class.Member member) {
                return this.order - ((Package.Class.Field)member).order;
            }
        }
        
        public class Method extends Package.Class.Member
        {
            Code code;
            
            public Method(int n, ConstantPool.DescriptorEntry descriptorEntry) {
                super(n, descriptorEntry);
                assert !(!descriptorEntry.isMethod());
                if (Package.Class.this.methods == null) {
                    Package.Class.this.methods = new ArrayList();
                }
                int add = Package.Class.this.methods.add(this);
                assert !(add == 0);
                return;
            }
            
            @Override
            public void trimToSize() {
                super.trimToSize();
                if (this.code != null) {
                    this.code.trimToSize();
                }
            }
            
            public int getArgumentSize() {
                return (Modifier.isStatic(this.flags) ? 0 : 1) + this.descriptor.typeRef.computeSize(true);
            }
            
            @Override
            public int compareTo(Package.Class.Member member) {
                return this.getDescriptor().compareTo(((Package.Class.Member)member).getDescriptor());
            }
            
            @Override
            public void strip(String anObject) {
                if ("Code".equals(anObject)) {
                    this.code = null;
                }
                if (this.code != null) {
                    this.code.strip(anObject);
                }
                super.strip(anObject);
            }
            
            @Override
            protected void visitRefs(int n, Collection<ConstantPool.Entry> collection) {
                super.visitRefs(n, collection);
                if (this.code != null) {
                    if (n == 0) {
                        collection.add(Package.getRefString("Code"));
                    }
                    this.code.visitRefs(n, collection);
                }
            }
        }
    }
    
    public final class File implements Comparable<Package.File>
    {
        String nameString;
        ConstantPool.Utf8Entry name;
        int modtime;
        int options;
        Package.Class stubClass;
        ArrayList<byte[]> prepend;
        ByteArrayOutputStream append;
        
        File(ConstantPool.Utf8Entry name) {
            this.modtime = 0;
            this.options = 0;
            this.prepend = new ArrayList();
            this.append = new ByteArrayOutputStream();
            this.name = name;
            this.nameString = name.stringValue();
        }
        
        File(String access$100) {
            this.modtime = 0;
            this.options = 0;
            this.prepend = new ArrayList();
            this.append = new ByteArrayOutputStream();
            access$100 = Package.access$100(access$100);
            this.name = Package.getRefString(access$100);
            this.nameString = this.name.stringValue();
        }
        
        public boolean isDirectory() {
            return this.nameString.endsWith("/");
        }
        
        public boolean isClassStub() {
            return (this.options & 0x2) != 0x0;
        }
        
        public Package.Class getStubClass() {
            assert !(!this.isClassStub());
            assert this.stubClass != null;
            return this.stubClass;
        }
        
        public boolean isTrivialClassStub() {
            return this.isClassStub() && this.name.stringValue().equals("") && (this.modtime == 0 || this.modtime == Package.this.default_modtime) && (this.options & 0xFFFFFFFD) == 0x0;
        }
        
        @Override
        public boolean equals(Object o) {
            return o != null && o.getClass() == Package.File.class && ((Package.File)o).nameString.equals(this.nameString);
        }
        
        @Override
        public int hashCode() {
            return this.nameString.hashCode();
        }
        
        @Override
        public int compareTo(Package.File file) {
            return this.nameString.compareTo(file.nameString);
        }
        
        @Override
        public String toString() {
            return this.nameString + "{" + (this.isClassStub() ? "*" : "") + (BandStructure.testBit(this.options, 1) ? "@" : "") + ((this.modtime == 0) ? "" : ("M" + this.modtime)) + ((this.getFileLength() == 0L) ? "" : ("[" + this.getFileLength() + "]")) + "}";
        }
        
        public java.io.File getFileName() {
            return this.getFileName(null);
        }
        
        public java.io.File getFileName(java.io.File parent) {
            return new java.io.File(parent, this.nameString.replace('/', java.io.File.separatorChar));
        }
        
        public void addBytes(byte[] array) {
            this.addBytes(array, 0, array.length);
        }
        
        public void addBytes(byte[] b, int off, int len) {
            if ((this.append.size() | len) << 2 < 0) {
                this.prepend.add(this.append.toByteArray());
                this.append.reset();
            }
            this.append.write(b, off, len);
        }
        
        public long getFileLength() {
            long n = 0L;
            if (this.prepend == null || this.append == null) {
                return 0L;
            }
            Iterator<byte[]> iterator = this.prepend.iterator();
            while (iterator.hasNext()) {
                n += (long)((byte[])iterator.next()).length;
            }
            return n + (long)this.append.size();
        }
        
        public void writeTo(OutputStream out) throws IOException {
            if (this.prepend == null || this.append == null) {
                return;
            }
            Iterator<byte[]> iterator = this.prepend.iterator();
            while (iterator.hasNext()) {
                out.write((byte[])iterator.next());
            }
            this.append.writeTo(out);
            return;
        }
        
        public void readFrom(InputStream inputStream) throws IOException {
            byte[] b = new byte[65536];
            int read;
            while ((read = inputStream.read(b)) > 0) {
                this.addBytes(b, 0, read);
            }
        }
        
        public InputStream getInputStream() {
            ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(this.append.toByteArray());
            if (this.prepend.isEmpty()) {
                return byteArrayInputStream;
            }
            ArrayList<Object> c = new ArrayList(this.prepend.size() + 1);
            Iterator<byte[]> iterator = this.prepend.iterator();
            while (iterator.hasNext()) {
                c.add(new ByteArrayInputStream((byte[])iterator.next()));
            }
            c.add(byteArrayInputStream);
            return new SequenceInputStream(Collections.enumeration(c));
        }
        
        protected void visitRefs(int n, Collection<ConstantPool.Entry> collection) {
            assert this.name != null;
            collection.add(this.name);
            return;
        }
    }
    
    static class InnerClass implements Comparable<Package.InnerClass>
    {
        final ConstantPool.ClassEntry thisClass;
        final ConstantPool.ClassEntry outerClass;
        final ConstantPool.Utf8Entry name;
        final int flags;
        final boolean predictable;
        
        InnerClass(ConstantPool.ClassEntry thisClass, ConstantPool.ClassEntry outerClass, ConstantPool.Utf8Entry name, int flags) {
            this.thisClass = thisClass;
            this.outerClass = outerClass;
            this.name = name;
            this.flags = flags;
            this.predictable = this.computePredictable();
        }
        
        private boolean computePredictable() {
            String[] innerClassName = Package.parseInnerClassName(this.thisClass.stringValue());
            if (innerClassName == null) {
                return false;
            }
            String s = innerClassName[0];
            String s2 = innerClassName[2];
            String s3 = (this.name == null) ? null : this.name.stringValue();
            String s4 = (this.outerClass == null) ? null : this.outerClass.stringValue();
            return s2 == s3 && s == s4;
        }
        
        @Override
        public boolean equals(Object o) {
            if (o == null || o.getClass() != Package.InnerClass.class) {
                return false;
            }
            Package.InnerClass innerClass = (Package.InnerClass)o;
            return eq(this.thisClass, innerClass.thisClass) && eq(this.outerClass, innerClass.outerClass) && eq(this.name, innerClass.name) && this.flags == innerClass.flags;
        }
        
        private static boolean eq(Object o, Object obj) {
            return (o == null) ? (obj == null) : o.equals(obj);
        }
        
        @Override
        public int hashCode() {
            return this.thisClass.hashCode();
        }
        
        @Override
        public int compareTo(Package.InnerClass innerClass) {
            return this.thisClass.compareTo(innerClass.thisClass);
        }
        
        protected void visitRefs(int n, Collection<ConstantPool.Entry> collection) {
            collection.add(this.thisClass);
            if (n == 0 || !this.predictable) {
                collection.add(this.outerClass);
                collection.add(this.name);
            }
        }
        
        @Override
        public String toString() {
            return this.thisClass.stringValue();
        }
    }
    
    protected static final class Version
    {
        public final short major;
        public final short minor;
        
        private Version(short major, short minor) {
            this.major = major;
            this.minor = minor;
        }
        
        @Override
        public String toString() {
            return (int)this.major + "." + (int)this.minor;
        }
        
        @Override
        public boolean equals(Object o) {
            return o instanceof Package.Version && this.major == ((Package.Version)o).major && this.minor == ((Package.Version)o).minor;
        }
        
        public int intValue() {
            return (this.major << 16) + this.minor;
        }
        
        @Override
        public int hashCode() {
            return (this.major << 16) + 7 + this.minor;
        }
        
        public static Package.Version of(int n, int n2) {
            return new Package.Version((short)n, (short)n2);
        }
        
        public static Package.Version of(byte[] array) {
            return new Package.Version((short)((array[2] & 0xFF) << 8 | (array[3] & 0xFF)), (short)((array[0] & 0xFF) << 8 | (array[1] & 0xFF)));
        }
        
        public static Package.Version of(int n) {
            return new Package.Version((short)(n >>> 16), (short)n);
        }
        
        public static Package.Version makeVersion(PropMap propMap, String s) {
            int integer = propMap.getInteger("com.sun.java.util.jar.pack." + s + ".minver", -1);
            int integer2 = propMap.getInteger("com.sun.java.util.jar.pack." + s + ".majver", -1);
            return (integer >= 0 && integer2 >= 0) ? of(integer2, integer) : null;
        }
        
        public byte[] asBytes() {
            return new byte[] { (byte)(this.minor >> 8), (byte)this.minor, (byte)(this.major >> 8), (byte)this.major };
        }
        
        public int compareTo(Package.Version version) {
            return this.intValue() - version.intValue();
        }
        
        public boolean lessThan(Package.Version version) {
            return this.compareTo(version) < 0;
        }
        
        public boolean greaterThan(Package.Version version) {
            return this.compareTo(version) > 0;
        }
    }
}
